/*
 * arch/arch/mach-ixp4xx/fhd3lan-pci.c
 *
 * PCI setup routines for Freecom-Storage-Gateway FSG3 Platform
 *
 * Copyright (C) 2004 MontaVista Software, Inc.
 * Copyright (C) 2006 Freecom Technologies GmbH.
 *
 * Maintainer: Freecom Technologies <support@freecom.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */

#include <linux/kernel.h>
#include <linux/pci.h>
#include <linux/init.h>
#include <linux/irq.h>

#include <asm/mach-types.h>
#include <asm/hardware.h>
#include <asm/irq.h>

#include <asm/mach/pci.h>

extern void ixp4xx_pci_preinit(void);
extern int ixp4xx_setup(int nr, struct pci_sys_data *sys);
extern struct pci_bus *ixp4xx_scan_bus(int nr, struct pci_sys_data *sys);

void __init fsg3_pci_preinit(void)
{
        set_irq_type(FSG3_PCI_INTA_IRQ, IRQT_LOW);
        set_irq_type(FSG3_PCI_INTB_IRQ, IRQT_LOW);
        set_irq_type(FSG3_PCI_INTC_IRQ, IRQT_LOW);

	ixp4xx_pci_preinit();
}

static int __init fsg3_map_irq(struct pci_dev *dev, u8 slot, u8 pin)
{
        static int pci_irq_table[FSG3_PCI_IRQ_LINES] = {
                FSG3_PCI_INTC_IRQ,
                FSG3_PCI_INTB_IRQ,
                FSG3_PCI_INTA_IRQ,
        };

        int irq = -1;
	slot = slot-11;

        if (slot >= 1 && slot <= FSG3_PCI_MAX_DEV &&
                pin >= 1 && pin <= FSG3_PCI_IRQ_LINES) {
                irq = pci_irq_table[(slot-1)];
        }
        printk("%s: Mapped slot %d pin %d to IRQ %d\n", __FUNCTION__,slot, pin, irq);

        return irq;
}

struct hw_pci fsg3_pci __initdata = {
	.nr_controllers = 1,
	.preinit =        fsg3_pci_preinit,
	.swizzle =        pci_std_swizzle,
	.setup =          ixp4xx_setup,
	.scan =           ixp4xx_scan_bus,
	.map_irq =        fsg3_map_irq,
};

int __init fsg3_pci_init(void)
{
	if (machine_is_freecom_fsg3())
		pci_common_init(&fsg3_pci);
	return 0;
}

subsys_initcall(fsg3_pci_init);
