/*
 * arch/arm/mach-ixp4xx/fsg3-setup.c
 *
 * FSG3/FHD3WLAN board-setup 
 *
 * Copyright (C) 2003-2004 MontaVista Software, Inc.
 * Copyright (C) 2004-2006 Freecom Technologies GmbH.
 *
 * Author: Freecom - CR
 */

#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/device.h>
#include <linux/serial.h>
#include <linux/tty.h>
#include <linux/serial_8250.h>
#include <linux/slab.h>
#include <linux/mtd/mtd.h>
#include <linux/leds.h>

#include <asm/types.h>
#include <asm/setup.h>
#include <asm/memory.h>
#include <asm/hardware.h>
#include <asm/irq.h>
#include <asm/mach-types.h>
#include <asm/mach/arch.h>
#include <asm/mach/flash.h>

#include <net/maclist.h>


/*
 * Xscale UART registers are 32 bits wide with only the least
 * significant 8 bits having any meaning.  From a configuration
 * perspective, this means 2 things...
 *
 *   Setting .regshift = 2 so that the standard 16550 registers
 *   line up on every 4th byte.
 *
 *   Shifting the register start virtual address +3 bytes when
 *   compiled big-endian.  Since register writes are done on a
 *   single byte basis, if the shift isn't done the driver will
 *   write the value into the most significant byte of the register,
 *   which is ignored, instead of the least significant.
 */

#ifdef  __ARMEB__
#define REG_OFFSET      3
#else
#define REG_OFFSET      0
#endif

/*
 * FSG3 offers both chipset serial ports
 */
static struct flash_platform_data fsg3_flash_data = {
	.map_name	= "cfi_probe",
	.width		= 2,
};

static struct resource fsg3_flash_resource = {
	.flags		= IORESOURCE_MEM,
};

static struct platform_device fsg3_flash = {
	.name		= "IXP4XX-Flash",
	.id		= 0,
	.dev.platform_data = &fsg3_flash_data,
	.num_resources	= 1,
	.resource	= &fsg3_flash_resource,
};

static struct ixp4xx_i2c_pins fsg3_i2c_gpio_pins = {
	.sda_pin	= FSG3_SDA_PIN,
	.scl_pin	= FSG3_SCL_PIN,
};

#ifdef CONFIG_LEDS_IXP4XX
static struct resource fsg3_led_resources[] = {
	{
		.name		= "sync",
		.start		= FSG3_LED_STARTUP,
		.end		= FSG3_LED_STARTUP,
		.flags		= 0x80,   // bit 7
	},
	{
		.name		= "ring",
		.start		= FSG3_LED_RING,
		.end		= FSG3_LED_RING,
		.flags		= 0x20,   // bit 5
	},
	{
		.name		= "USB",
		.start		= FSG3_LED_USB,
		.end		= FSG3_LED_USB,
		.flags		= 0x10,   // bit 4
	},
	{
		.name		= "SATA",
		.start		= FSG3_LED_SATA,
		.end		= FSG3_LED_SATA,
		.flags		= 0x04,   // bit 2
	},
	{
		.name		= "WAN",
		.start		= FSG3_LED_WAN,
		.end		= FSG3_LED_WAN,
		.flags		= 0x02,   // bit 1
	},
	{
		.name		= "WLAN",
		.start		= FSG3_LED_WLAN,
		.end		= FSG3_LED_WLAN,
		.flags		= 0x01,   // bit 0
	},
};

static struct platform_device fsg3_leds = {
	.name			= "IXP4XX-GPIO-LED",
	.id			= -1,
	.num_resources		= ARRAY_SIZE(fsg3_led_resources),
	.resource		= fsg3_led_resources,
};
#endif

static struct platform_device fsg3_i2c_controller = {
	.name		= "IXP4XX-I2C",
	.id		= 0,
	.dev.platform_data = &fsg3_i2c_gpio_pins,
	.num_resources	= 0,
};

static struct resource fsg3_uart_resources[] = {
        {
                .start          = IXP4XX_UART1_BASE_PHYS,
                .end            = IXP4XX_UART1_BASE_PHYS + 0x0fff,
                .flags          = IORESOURCE_MEM,
        },
        {
                .start          = IXP4XX_UART2_BASE_PHYS,
                .end            = IXP4XX_UART2_BASE_PHYS + 0x0fff,
                .flags          = IORESOURCE_MEM,
        }
};

static struct plat_serial8250_port fsg3_uart_data[] = {
        {
                .mapbase        = IXP4XX_UART1_BASE_PHYS,
                .membase        = (char *)IXP4XX_UART1_BASE_VIRT + REG_OFFSET,
                .irq            = IRQ_IXP4XX_UART1,
                .flags          = UPF_BOOT_AUTOCONF | UPF_SKIP_TEST,
                .iotype         = UPIO_MEM,
                .regshift       = 2,
                .uartclk        = IXP4XX_UART_XTAL,
/*		.line		= 0,
		.type		= PORT_XSCALE,
		.fifosize	= 32
*/
        },
        {
                .mapbase        = IXP4XX_UART2_BASE_PHYS,
                .membase        = (char *)IXP4XX_UART2_BASE_VIRT + REG_OFFSET,
                .irq            = IRQ_IXP4XX_UART2,
                .flags          = UPF_BOOT_AUTOCONF | UPF_SKIP_TEST,
                .iotype         = UPIO_MEM,
                .regshift       = 2,
                .uartclk        = IXP4XX_UART_XTAL,
/*		.line		= 1,
		.type		= PORT_XSCALE,
		.fifosize	= 32
*/
        },
        { }
};

static struct platform_device fsg3_uart = {
        .name                   = "serial8250",
        .id                     = PLAT8250_DEV_PLATFORM,
        .dev.platform_data      = fsg3_uart_data,
        .num_resources          = 2,
        .resource               = fsg3_uart_resources,
};

static struct platform_device *fsg3_devices[] __initdata = {
	&fsg3_i2c_controller,
	&fsg3_flash,
	&fsg3_uart,
#ifdef CONFIG_LEDS_IXP4XX
	&fsg3_leds,
#endif
};

#ifdef CONFIG_MACLIST
static void fsg3_flash_add(struct mtd_info *mtd)
{
	/* When the RedBoot partition is added the MAC address is read from it. */
	if (strcmp(mtd->name, "RedBoot config") == 0) {
		size_t retlen;
		u_char mac[6];

		/* The MAC is at a known offset... */
		/* TODO --> find npe_eth0 and ..1 instead of hardcoded offset and inc) */
		if (mtd->read(mtd, 0x0422, 6, &retlen, mac) == 0 && retlen == 6) {
			printk(KERN_INFO "FSG3-MAC for eth0: %.2x:%.2x:%.2x:%.2x:%.2x:%.2x\n",
				mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
			maclist_add(mac);
			mac[5]++;
			printk(KERN_INFO "FSG3-MAC for eth1: %.2x:%.2x:%.2x:%.2x:%.2x:%.2x\n",
				mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
			maclist_add(mac);
		} else {
			printk(KERN_ERR "FGS3-MAC: read failed\n");
		}
	}
}

static void fsg3_flash_remove(struct mtd_info *mtd)
{
	/* Nothing to do on remove at present. */
}

static struct mtd_notifier fsg3_flash_notifier = {
	.add = fsg3_flash_add,
	.remove = fsg3_flash_remove,
};
#endif

static void fsg3_power_off(void)
{
	/* EL-Ring and LED's can be switched off here. */
        printk(KERN_INFO "EL-Ring and LED's can be switched off here.\n");
}

static void __init fsg3_init(void)
{
	/* The flash has an ethernet MAC embedded in it which we need,
	 * that is all this notifier does. */

	ixp4xx_sys_init();

	fsg3_flash_resource.start = FSG3_FLASH_BASE;
	fsg3_flash_resource.end = FSG3_FLASH_BASE + FSG3_FLASH_SIZE;

        *IXP4XX_EXP_CS0 |= IXP4XX_FLASH_WRITABLE;
        *IXP4XX_EXP_CS1 = *IXP4XX_EXP_CS0;

	/* Configure CS2 for operation, 8bit and writable will do */
	*IXP4XX_EXP_CS2 = 0xbfff0002;

	pm_power_off = fsg3_power_off;

	platform_add_devices(fsg3_devices, ARRAY_SIZE(fsg3_devices));

#ifdef CONFIG_MACLIST
	register_mtd_user(&fsg3_flash_notifier);
#endif
}

MACHINE_START(FREECOM_FSG3, "FSG3 - Freecom Storage Gateway")
        /* Maintainer: Freecom Technologies GmbH */
        .phys_io        = IXP4XX_PERIPHERAL_BASE_PHYS,
        .io_pg_offst    = ((IXP4XX_PERIPHERAL_BASE_VIRT) >> 18) & 0xFFFC,
        .boot_params    = 0x00000100,
        .map_io         = ixp4xx_map_io,
        .init_irq       = ixp4xx_init_irq,
        .timer          = &ixp4xx_timer,
        .init_machine   = fsg3_init,
MACHINE_END
