#ifndef _MACLIST_H
#define _MACLIST_H 1
/*
 * Interfaces to the MAC repository
 *
 * Copyright (C) 2005 John Bowler
 * Author: John Bowler <jbowler@acm.org>
 * Maintainers: http://www.nslu2-linux.org/
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 */

/*
 * Add a single entry, returns 0 on success else an error
 * code.  Allocates memory, claims and releases a write
 * lock.
 */
extern int maclist_add(const u8 id_to_add[6]);

/*
 * Return the current entry count, claims and releases a
 * read lock.
 */
extern int maclist_count(void);

/*
 * Return the ID from the given entry.  Always succeeds.
 * Claims and releases a write lock.
 *
 * If any entry has not been allocated for this key one
 * is allocated.  If there are no remaining unallocated
 * entries a new one is created.
 *
 * If the value of the key is less than maclist_count()
 * the entry indexed by the key (i.e. for key 'n' the
 * n'th entry starting at 0) will be returned if available.
 * Otherwise the entry to be returned will be unpredictable
 * but consistent for a given value of maclist_count().
 */
extern void maclist_read(u8 (*buffer_for_id)[6],
		u32 key_of_entry_to_return);

/*
 * See the implementation in drivers/net/maclist.c for
 * more information.
 */
#endif /*_MACLIST_H*/
